#include <stdlib.h> // pour la macro NULL

#include "ville.h"
#include "Graphe.h"
//***********************************************************************************************************
// RELIEES: teste si les 2 villes passes en paramtre sont bien dans la meme composante connexe.
bool Graphe::reliees(const Ville * const v1, const Ville * const v2)
{
  // initialisation:
  arrivee= v2;
  atteintes.vide();
  // appel  la fonction rcursive:
  return butAtteint(v1);
}
//...........................................................................................................
// Fonction rcursive appele par reliees:
bool Graphe::butAtteint(const Ville * const v)
{
  if (v==arrivee)
    return true;

  atteintes.ajoute(*v);
  QList<Ville> voisines= VillesVoisines(v);

  voisines.first();

  while (voisines.current() != NULL)
    {
      // traitement:
      Ville * w= voisines.current();

      if (!atteintes.contient(*w) && butAtteint(w))
	      return true;
      
      voisines.next();
    }

  return false;
}
//***********************************************************************************************************
// COMPOSANTE: renvoie la liste des villes qui sont dans la meme composante connexe que v (v comprise).
QList<Ville> & Graphe::composante(const Ville * const v)
{
  // initialisation:
  atteintes.vide();
  listeVilles= new QList<Ville>;
  // appel  la fonction rcursive:
  descente(v);
  // renvoi du rsultat:
  return *listeVilles;
}
//...........................................................................................................
// Fonction appele par composante (et composantes):
void Graphe::descente(const Ville * const v)
{
  atteintes.ajoute(*v);
  listeVilles->append(v);

  QList<Ville> voisines= VillesVoisines(v);

  voisines.first();

  while (voisines.current() != NULL)
    {
      // traitement:
      Ville * w= voisines.current();

      if (!atteintes.contient(*w))
	descente(w);

      voisines.next();
    }  
}
//***********************************************************************************************************
// COMPOSANTES: renvoie la liste des composantes connexes du graphe (en utilisant descente).
QList <QList<Ville> > & Graphe::listeComposantes()
{
  QList <QList<Ville> > * listeComposantes= new QList<QList<Ville> >;

  listeComposantes->setAutoDelete(true);
  atteintes.vide();

  Ville * v= premVille();

  while (v!=NULL)
    {
      //trouver une ville non atteinte:
      while ((v!=NULL) && atteintes.contient(*v))
	v=villeSuiv(v);

      if (v!=NULL)
	{
	  // appeler descente sur v SANS VIDER L'ENSEMBLE ATTEINTES:
	  listeVilles= new QList<Ville>;
	  descente(v);
	  listeComposantes->append(listeVilles);
	}
    }

  return * listeComposantes;
}
//***********************************************************************************************************
//***********************************************************************************************************
//***********************************************************************************************************
QList<Ville> & Graphe::VillesVoisines(Ville * const v)
{
  QList<Route> routes= v->retVoisins();

  QList<Ville> * resultat= new QList<Ville>;

  for (int i=0; i<routes.count(); i++)
      if (*v == *(routes.at(i)->retextrem1()))
        resultat->append(routes.at(i)->retextrem2());
      else
	resultat->append(routes.at(i)->retextrem1());

  return *resultat;
}
//***********************************************************************************************************
//***********************************************************************************************************
//***********************************************************************************************************
ensemble::ensemble()
{
  contenu.setAutoDelete(false);
}

ensemble::~ensemble()
{
}

void ensemble::vide()
{
  contenu.clear();
}

bool ensemble::contient(const Ville v) const
{
  return contenu.find(v.rethash().data()) != NULL;
}

void ensemble::ajoute(const Ville v)
{
  contenu.insert(v.rethash().data(),&v);
}

unsigned int ensemble::cardinal()
{
  return contenu.count();
}
//***********************************************************************************************************
//***********************************************************************************************************
//***********************************************************************************************************
