//regroupement de Image0.cpp et Image2.cpp


#include <stdio.h>
#include <qpen.h>
#include "math.h"
#include "Image.h"

/////////////////////////////////CONSTRUCTEURS/////////////////////////////////////////////////
Image::Image(QMainWindow *parent, const char *name) : QWidget(parent, name)
{
  init(QPixmap());
  Initialisation();
}

Image::Image(const QPixmap &pmap, QWidget *parent, const char *name)
	 : QWidget(parent, name)
{
  init(pmap);
  Initialisation();
}

Image::Image(const QString &fileName, QWidget *parent, const char *name) : QWidget(parent, name)
{
  QPixmap	pmap;

  if (pmap.load(fileName)==false)
    {
      pmap.resize(0, 0);
      warning("Can't load file " + fileName);
    }

  init(pmap);
  Initialisation();
}



////////////////////////////////DESCTRUCTEUR//////////////////////////////////////////////////
Image::~Image()
{ ppix->end();
  paint->end();
}

bool Image::valide() const
{
  return imageSource && imageCourante && !imageSource->isNull() && !imageCourante->isNull() && etatOk;
}

void Image::charger(const QPixmap &pmap)
{
  if (selectionEnCours())
    arreterSelection();

  initParametresZoom(pmap);

  repaint(false);
}

void Image::charger(const QString &nomFichier)
{
  if (selectionEnCours())
    arreterSelection();

  QPixmap	pmap;

  if (pmap.load(nomFichier)==false)
    {
      pmap.resize(0, 0);
      warning("Image::load(const QString &): Can't load file " + nomFichier);
    }

  initParametresZoom(pmap);

  repaint(false);
}

void Image::charger(const char *nomFichier)
{
  charger(QString(nomFichier));
}

void Image::recharger()
{
  if (selectionEnCours())
    arreterSelection();

  if (valide())
    initParametresZoom(*imageSource);
}

// les fonctions de zoom sont en cours d'ameliorations
////////////////////////////ZOOMS////////////////////////////////////
float Image::coefZoom() const
{
  float sx,
	sy;

  calculerCoefZoom(sx, sy);
  return sx;
}

QPoint Image::position() const
{
  if(!valide())
    return QPoint(0,0);

  return QPoint(x0,y0);
}

QSize Image::maximumSize() const
{
  if (!valide())
    return QSize(0, 0);

  float	sx,
    	sy;
  calculerCoefZoom(sx, sy);

  int	largeurMax	= int(imageSource->width() *sx),
    	hauteurMax	= int(imageSource->height() *sy);

  return QSize(largeurMax, hauteurMax);
}

void Image::zoomBy(const QPoint &pos, float coef)
{
  if (!valide())
   {
     warning("Image::zoomBy(const QPoint&, float): Can't zoom a null image");
     return;
   }

  if (coef<=0)
    {
      warning("Image::zoomBy(const QPoint &, float): Can't zoom with coef <= 0");
      return;
    }

  if (coef==1)
    return;

  QRect		cadre;
  int		largeur,
    		hauteur;
  QSize		s;
  float		sx,
    		sy;
  QPoint	posGlobale	= pos;

  cadreImageCourante(cadre);
  calculerCoefZoom(sx, sy);

  largeur	= int(cadre.width() / coef);
  hauteur	= int(cadre.height() / coef);

  if (largeur<1)
    largeur	= 1;

  if (hauteur<1)
    hauteur	= 1;

  sx		= sx * coef;
  sy		= sy * coef;

  enGlobal(posGlobale);
  cadreCentre(posGlobale, QSize(largeur, hauteur), cadre);

  bool	redimWidget;

  s	= size();
  redimWidget	= ajusterImage(cadre, sx, sy, s);

  definirImageCourante(cadre, sx, sy);
  ajusterWidget();

  if (redimWidget)
    resize(s);

  repaint(false);
}

void Image::zoom(int x, int y, int w, int h)
{
  if (!valide())
    return;
  
  QRect		cadreLocal(x, y, w, h);

  if (cadreLocal.isEmpty())
    return;

  ajuster(cadreLocal);

  float		sx,
    		sy;

  calculerCoefZoom(sx, sy);

  sx	*= (float) imageCourante->width() / cadreLocal.width();
  sy	*= (float) imageCourante->height() / cadreLocal.height();

  QPoint	coinHautGauche(cadreLocal.topLeft()),
    		coinBasDroit(cadreLocal.bottomRight());

  enGlobal(coinHautGauche);
  enGlobal(coinBasDroit);

  QRect	cadreGlobal(coinHautGauche, coinBasDroit);
  QSize	s(size());
  bool	redimWidget;

  redimWidget	= ajusterImage(cadreGlobal, sx, sy, s);

  definirImageCourante(cadreGlobal, sx, sy);
  ajusterWidget();

  if (redimWidget)
    resize(s);

  repaint(false);
}

void Image::zoom(const QRect &zoomBox)
{
  zoom(zoomBox.left(), zoomBox.top(), zoomBox.width(), zoomBox.height());
}

void Image::zoomAbsolu(float coef)
{
  if (!valide())
    return;

  QRect		cadre;
  cadreImageCourante(cadre);

  QPoint	centre	= cadre.center();

  int	largeur	= int(imageCourante->width() / coef),
    	hauteur	= int(imageCourante->height() / coef);

  QSize	s(size());

  bool	redimWidget;

  cadreCentre(centre, QSize(largeur, hauteur), cadre);

  redimWidget	= ajusterImage(cadre, coef, coef, s);

  definirImageCourante(cadre, coef, coef);
  ajusterWidget();

  if (redimWidget)
    resize(s);
 
  repaint(false);
}

void Image::zoomAbsolu(int x, int y, int w, int h)
{
  if (!valide())
    return;

  QRect	cadre(x, y, w, h),
    	cadreImageSource(0, 0, imageSource->width(), imageSource->height());

  cadre	= cadreImageSource.intersect(cadre);

  if (cadre.isEmpty())
    return;

  int	largeur	= cadre.width(),
    	hauteur	= cadre.height();

  float	sx	= (float) imageCourante->width() / largeur,
    	sy	= (float) imageCourante->height() / hauteur;

  QSize	s(size());

  bool	redimWidget;

  redimWidget	= ajusterImage(cadre, sx, sy, s);
  definirImageCourante(cadre, sx, sy);
  ajusterWidget();

  if (redimWidget)
    resize(s);

  repaint(false);
}

void Image::zoomAbsolu(const QRect &zoomBox)
{
  zoomAbsolu(zoomBox.left(), zoomBox.top(), zoomBox.width(), zoomBox.height());
}

void Image::annulerZoom()
{
  QRect		cadre(x0Precedent, y0Precedent, largeurVirtuellePrecedente, hauteurVirtuellePrecedente);
  float		sx	= (float) largeurReellePrecedente / largeurVirtuellePrecedente,
    		sy	= (float) hauteurReellePrecedente / hauteurVirtuellePrecedente;
  QSize		s(size());
  bool		redimWidget;

  redimWidget	= ajusterImage(cadre, sx, sy, s);
  definirImageCourante(cadre, sx, sy);
  ajusterWidget();

  if (redimWidget)
    resize(s);

  repaint(false);
}

void Image::zoom1_4()
{
  zoomAbsolu(0.25);
}
void Image::zoom1_3()
{
  zoomAbsolu(1./3);
}

void Image::zoom1_2()
{
  zoomAbsolu(0.5);
}

void Image::zoom1_1()
{
  zoomAbsolu(1);
}

void Image::zoom2_1()
{
  zoomAbsolu(2);
}

void Image::zoom3_1()
{
  zoomAbsolu(3);
}

void Image::zoom4_1()
{
  zoomAbsolu(4);
}

void Image::changeOutils(int i)
{
  int o=outils;
  outils=i;
  if (o==5) 
    refresh();
}

void Image::outilsZoom()
{
  changeOutils(0);
  QBitmap	imageCurseur,
    		maskCurseur;

  imageCurseur.load("mini-zoom.xpm");
  maskCurseur.load("mini-zoom-mask.xpm");

  QCursor curseur(imageCurseur, maskCurseur, 0, 0);
  setCursor(curseur);
}

void Image::outilsDeplace()
{
  changeOutils(2);
  setCursor(ArrowCursor);
}
void Image::outilsSelection()
{
  changeOutils(3);
  setCursor(ArrowCursor);
}

void  Image::outilsCreation()
{
  changeOutils(4);
  setCursor(ArrowCursor);
}


void  Image::outilsInfo()
{
  changeOutils(5);
  setCursor(ArrowCursor);
}

void Image::outilsChemin()
{
  changeOutils(6);
  setCursor(ArrowCursor);
}


void Image::scroll(int dx, int dy)
{
  if (!valide())
    return;

  QPoint	p(dx, dy);
  QRect		cadre;
  float		sx,
    		sy;
  int		gauche,
    		haut,
    		xMax,
    		yMax;
  QSize		tailleMax;
  QWMatrix	m;

  calculerCoefZoom(sx, sy);
  m.scale(1/sx, 1/sy);
  p	= m.map(p);

  tailleMax	= maximumSize();

  xMax	= int((tailleMax.width() - width()) / sx);
  yMax	= int((tailleMax.height() - height()) / sy);

  cadreImageCourante(cadre);
  cadre.moveTopLeft(cadre.topLeft() + p);

  gauche	= cadre.left();
  haut		= cadre.top();

  if (xMax<0)
    xMax	= 0;

  if (yMax<0)
    yMax	= 0;

  if (gauche<0)
    gauche	= 0;

  if (haut<0)
    haut	= 0;

  if (gauche>xMax)
    gauche	= xMax;

  if (haut>=yMax)
    haut	= yMax;

  cadre.moveTopLeft(QPoint(gauche, haut));

  x0	= cadre.left();
  y0	= cadre.top();
  imageAJour	= false;
  //  definirImageCourante(cadre, sx, sy);

  repaint(false);
}

void Image::scrollAbsolu(int x, int y)
{
  if (!valide())
    return;

  float	sx,
    	sy;
  QRect	cadre;
  cadreImageCourante(cadre);
  calculerCoefZoom(sx, sy);

  QSize	tailleMax;
  int	xMax,
    	yMax;

  tailleMax	= maximumSize();

  xMax	= int((tailleMax.width() - width() - 1) / sx);
  yMax	= int((tailleMax.height() - height() - 1) / sy);

  if (xMax<0)
    xMax	= 0;

  if (yMax<0)
    yMax	= 0;

  if (x<0)
    x	= 0;

  if (x>xMax)
    x	= xMax;

  if (y<0)
    y	= 0;

  if (y>yMax)
    y	= yMax;

  cadre.moveTopLeft(QPoint(x, y));

  x0	= cadre.left();
  y0	= cadre.top();
  imageAJour	= false;
  //  definirImageCourante(cadre, sx, sy);

  repaint(false);
}

void Image::cadreCentre(const QPoint &pos, const QSize &s, QRect &cadre) const
{
  if (!valide())
    return;

  int 	largeur	= s.width(),
	hauteur = s.height();

  if (largeur>imageSource->width())
	largeur = imageSource->width();

  if (hauteur>imageSource->height())
    	hauteur = imageSource->height();

  QRect	cadreGlobal(0, 0, largeur, hauteur);
  cadreGlobal.moveCenter(pos);

  int 	haut 	= cadreGlobal.top(),
    	bas,
    	gauche	= cadreGlobal.left(),
	droit;

  if (gauche<0)
    	gauche	= 0;

  if (haut<0)
	haut	= 0;

  cadreGlobal.moveTopLeft(QPoint(gauche, haut));

  droit	= cadreGlobal.right();
  bas	= cadreGlobal.bottom();

  if (droit>=imageSource->width())
	droit	= imageSource->width() - 1;

  if (bas>=imageSource->height())
	bas	= imageSource->height() - 1;

  cadreGlobal.moveBottomRight(QPoint(droit, bas));

  cadre = cadreGlobal;
}

void Image::calculerImageCourante()
{
  if (!valide())
    return;

  if (imageCouranteAJour())
    return;

  QRect	cadre;
  float	sx,
    	sy;

  cadreImageCourante(cadre);
  calculerCoefZoom(sx, sy);

  int	largeur	= cadre.width() + kMargeCalculImage,
    	hauteur	= cadre.height() + kMargeCalculImage;

  imageCourante->resize(largeur, hauteur);
  bitBlt(imageCourante, 0, 0, imageSource, cadre.left(), cadre.top(), largeur, hauteur);

  QWMatrix	m;
  m.scale(sx, sy);

  *imageCourante	= imageCourante->xForm(m);
  imageCourante->resize(size());

  imageAJour	= true;
}

void Image::calculerCoefZoom(float &sx, float &sy) const
{
  if (!valide())
    {
      sx = 0;
      sy = 0;

      return;
    }

  sx	= (float) largeurReelle / largeurVirtuelle;
  sy	= (float) hauteurReelle / hauteurVirtuelle;

  if (sx < sy)
    sy	= sx;
  else
    sx	= sy;
}

void Image::definirImageCourante(const QRect &cadre, float sx, float sy)
{
  if (!valide())
    return;

  if (sx<=0 || sy<=0)
    {
      warning("Image::definirImageCourante(const QRect &, float, float): les coefficients de zoom doivent etre > 0");
      return;
    }

  if (sx < sy)
    sy	= sx;
  else
    sx	= sy;

  float	sxPrecedent,
    	syPrecedent;
  calculerCoefZoom(sxPrecedent, syPrecedent);

  long	largeurImageMax			= long(imageSource->width() * sx),
    	hauteurImageMax			= long(imageSource->height() * sy),
    	largeurImageMaxPrecedente	= long(imageSource->width() * sxPrecedent),
    	hauteurImageMaxPrecedente	= long(imageSource->height() * syPrecedent);

  if (largeurImageMax>SHRT_MAX || hauteurImageMax>SHRT_MAX)
    {
      warning("Image::definirImageCourante(const QRect &, float, float): l'image courante definie depasse les limites autorisees\nL'image courante ne sera pas modifiee");
      return;
    }

  QRect	cadreImageSource(0, 0, imageSource->width(), imageSource->height()),
    	nouveauCadre	= cadre;

  nouveauCadre	= cadreImageSource.intersect(nouveauCadre);

  if (nouveauCadre.isEmpty())
    return;

  long	largeurImage,
    	hauteurImage;

  largeurImage	= int(nouveauCadre.width() * sx);
  hauteurImage	= int(nouveauCadre.height() * sy);

  if (largeurImage<kTailleImageMin || hauteurImage<kTailleImageMin)
    {
      warning("Image::definirImageCourante(const QRect &, float, float): l'image courante definie depasse les limites autorisees\nL'image courante ne sera pas modifiee");
      return;
    }

  x0Precedent			= x0;
  y0Precedent			= y0;
  largeurVirtuellePrecedente	= largeurVirtuelle;
  hauteurVirtuellePrecedente	= hauteurVirtuelle;

  largeurReellePrecedente	= largeurReelle;
  hauteurReellePrecedente	= hauteurReelle;

  x0				= nouveauCadre.left();
  y0				= nouveauCadre.top();
  largeurVirtuelle		= nouveauCadre.width();
  hauteurVirtuelle		= nouveauCadre.height();

  largeurReelle			= largeurImage;
  hauteurReelle			= hauteurImage;

  imageAJour			= false;
  
  if (x0Precedent!=x0 || y0Precedent!=y0)
    emit imageScrollee(x0, y0);
  
  if (largeurImageMax!=largeurImageMaxPrecedente || hauteurImageMax!=hauteurImageMaxPrecedente)
    emit tailleMaxChangee(int(largeurImageMax), int(hauteurImageMax));
  
  if (sx!=sxPrecedent || sy!=syPrecedent)
    emit coefZoomChange(coefZoom());
}

void Image::cadreImageCourante(QRect &cadre) const
{
  if (!valide())
    return;

  cadre.setLeft(x0);
  cadre.setTop(y0);
  cadre.setWidth(largeurVirtuelle);
  cadre.setHeight(hauteurVirtuelle);
}

bool Image::imageCouranteAJour() const
{
  return imageAJour;
}

//////////////////////////SELECTIONS///////////////////////////////////////
bool Image::cadreSelectionAffiche() const
{
  return cadreAffiche;
}

void Image::placerCadreSelection(const QPoint &coinOppose)
{
  if (!selectionEnCours())
    return;

  if (coinOppose==positionFinSelection)
    return;

  if (cadreSelectionAffiche())
    effacerCadreSelection(paint);

  QPoint	p(coinOppose);

  ajuster(p);
  positionFinSelection 	= p;

  QPoint	global	= p;
  enGlobal(global);

  afficherCadreSelection(paint);
}

void Image::restaurerSousCadreSelection(QPainter *p)
{
  if (!selectionEnCours())
    return;

  if (cadreSelectionAffiche())
    return;

  QRect		cadre;
  int		gauche,
    		droite,
    		haut,
    		bas;

  cadreSelection(cadre);
  gauche	= cadre.left();
  droite	= cadre.right();
  haut		= cadre.top();
  bas		= cadre.bottom();

  if (p!=NULL)
    {
      RasterOp	op	= p->rasterOp();
      QPen	penInfo	= p->pen();

      p->setPen(QPen(QColor(black), 1, SolidLine));
      p->setRasterOp(NotROP);
      p->drawRect(cadre);

      p->setPen(penInfo);
      p->setRasterOp(op);
    }
  else
    {
      QPainter	p(this);

      p.setPen(QPen(QColor(black), 1, SolidLine));
      p.setRasterOp(NotROP);
      p.drawRect(cadre);
    }
}

bool Image::cadreSelection(QRect &cadre) const
{
  if (!selectionEnCours())
    return false;

  int		gauche,
    		droite,
    		haut,
    		bas;

    if (positionDebutSelection.x() < positionFinSelection.x())
    {
      gauche	= positionDebutSelection.x();
      droite	= positionFinSelection.x();
    }
  else
    {
      gauche	= positionFinSelection.x();
      droite	= positionDebutSelection.x();
    }
  
  if (positionDebutSelection.y() < positionFinSelection.y())
    {
      haut	= positionDebutSelection.y();
      bas	= positionFinSelection.y();
    }
  else
    {
      haut	= positionFinSelection.y();
      bas	= positionDebutSelection.y();
    }

  cadre.setLeft(gauche);
  cadre.setRight(droite);
  cadre.setTop(haut);
  cadre.setBottom(bas);

  return true;
}

void Image::afficherCadreSelection(QPainter *p)
{
  if (!selectionEnCours())
    return;

  if (cadreSelectionAffiche())
    return;

  QRect	cadre;

  cadreSelection(cadre);

  if (p!=NULL)
    {
      RasterOp	op	= p->rasterOp();
      QPen	penInfo	= p->pen();

      p->setPen(QPen(QColor(black), 1, SolidLine));
      p->setRasterOp(NotROP);

      p->drawRect(cadre);

      p->setPen(penInfo);
      p->setRasterOp(op);
    }
  else
    {
      QPainter	p(this);

      p.setPen(QPen(QColor(black), 1, SolidLine));
      p.setRasterOp(NotROP);

      p.drawRect(cadre);
    }

  cadreAffiche	= true;
}

void Image::effacerCadreSelection(QPainter *p)
{
  if (!selectionEnCours())
    return;

  if (!cadreSelectionAffiche())
    return;

  cadreAffiche	= false;

  restaurerSousCadreSelection(paint);
}

bool Image::enGlobal(QPoint &pos) const
{
  if (!valide())
    return false;

  QWMatrix	mat;
  float		sx,
    		sy;
  QPoint	origine(x0, y0);	

  calculerCoefZoom(sx, sy);

  mat.scale(1/sx, 1/sy);
  pos	= mat.map(pos);

  pos	= pos + origine;

  return true;
}

bool Image::enLocal(QPoint &pos) const
{
  if (!valide())
    return false;

  QWMatrix	mat;
  float		sx,
    		sy;
  QPoint	origine(x0, y0);

  calculerCoefZoom(sx, sy);

  pos	= pos - origine;
  mat.scale(sx, sy);
  pos	= mat.map(pos);

  return true;
}
///////////////////////////////AJUSTEMENTS////////////////////////

bool Image::ajusterImage(QRect &cadre, float sx, float sy, QSize &s) const
{
  if (!valide())
    return false;

  if (s.isNull())
    return false;

  if (sx < sy)
    sy	= sx;
  else
    sx	= sy;

  long	largeurImageMax	= long(imageSource->width() * sx),
    	hauteurImageMax	= long(imageSource->height() * sy);

  if (largeurImageMax>SHRT_MAX || hauteurImageMax>SHRT_MAX)
    {
      warning("Image::ajuster(QRect &, float, float, QSize &): l'image courante definie depasse les limites autorisees\nL'image courante ne sera pas modifiee");
      return false;
    }

  QWMatrix	LG,
    		GL;
  QRect		cadreImageSource(0, 0, imageSource->width(), imageSource->height());

  LG.scale(1/sx, 1/sy);
  GL.scale(sx, sy);

  QRect	cadreTemp;
  long	largeurImage,
    	hauteurImage;

  cadreTemp	= cadreImageSource.intersect(cadre);

  largeurImage	= int(cadreTemp.width() * sx);
  hauteurImage	= int(cadreTemp.height() * sy);

  if (largeurImage<kTailleImageMin || hauteurImage<kTailleImageMin)
    {
      warning("Image::ajuster(QRect &, float, float, QSize &): l'image courante definie depasse les limites autorisees\nL'image courante ne sera pas modifiee");
      return false;
    }

  cadre	= cadreTemp;

  QPoint	cadreBasDroit(cadre.bottomRight() - cadre.topLeft()),
    		limiteBasDroit(s.width() - 1, s.height() - 1);

  cadreBasDroit	= GL.map(cadreBasDroit);

  int	dx	= limiteBasDroit.x() - cadreBasDroit.x(),
    	dy	= limiteBasDroit.y() - cadreBasDroit.y();

  if (dx<0)
    dx	= 0;

  if (dy<0)
    dy	= 0;

  QPoint	imageSourceBasDroit(cadreImageSource.bottomRight()),
    		origine(cadre.topLeft());
  origine		= GL.map(origine);
  imageSourceBasDroit	= GL.map(imageSourceBasDroit) - origine;

  int	maxDx	= imageSourceBasDroit.x() - cadreBasDroit.x(),
    	maxDy	= imageSourceBasDroit.y() - cadreBasDroit.y(),
    	diffDx	= dx - maxDx,
    	diffDy	= dy - maxDy;

  if (diffDx > 0)
    dx		= maxDx;
  else
    diffDx	= 0;

  if (diffDy > 0)
    dy		= maxDy;
  else
    diffDy	= 0;

  QPoint	decalage(dx, dy);
  decalage	= LG.map(decalage);

  cadre.setRight(cadre.right() + decalage.x());
  cadre.setBottom(cadre.bottom() + decalage.y());

  if (diffDx>0 || diffDy>0)
    {
      maxDx	= origine.x();
      maxDy	= origine.y();

      if (diffDx > maxDx)
	{
	  dx		= maxDx;
	  diffDx	-= maxDx;
	}
      else
	{
	  dx		= diffDx;
	  diffDx	= 0;
	}

      if (diffDy > maxDy)
	{
	  dy		= maxDy;
	  diffDy	-= maxDy;
	}
      else
	{
	  dy		= diffDy;
	  diffDy	= 0;
	}

      decalage	= QPoint(dx, dy);
      decalage	= LG.map(decalage);

      cadre.setLeft(cadre.left() - decalage.x());
      cadre.setTop(cadre.top() - decalage.y());
    }

  if (diffDx>0 || diffDy>0)
    {
      s	-= QSize(diffDx, diffDy);
      return true;
    }


  return false;
}

void Image::ajusterWidget()
{
  if (!valide())
    return;

  float	sx,
    	sy;
  int	largeurMax,
    	hauteurMax;

  calculerCoefZoom(sx, sy);

  largeurMax	= int(imageSource->width() * sx);
  hauteurMax	= int(imageSource->height() * sy);

  setMaximumSize(largeurMax, hauteurMax);
}

void Image::ajuster(QRect &r) const
{
  if (!valide())
    return;

  QRect	cadre(0, 0, imageCourante->width(), imageCourante->height());

  r   = cadre.intersect(r);
}

void Image::ajuster(QPoint &p) const
{
  if (!valide())
    return;

  int	gauche	= 0,
    	droite	= gauche + imageCourante->width(),
    	haut	= 0,
    	bas	= haut + imageCourante->height(),
    	x	= p.x(),
    	y	= p.y();

  if (x<gauche)
    x	= gauche;

  if (x>=droite)
    x	= droite - 1;

  if (y<haut)
    y	= haut;

  if (y>=bas)
    y	= bas - 1;

  p.setX(x);
  p.setY(y);
}

bool Image::selectionEnCours() const
{
  return _selectionEnCours;
}

void Image::commencerSelection(const QPoint &pos)
{
  if (selectionEnCours())
    arreterSelection();

  tempsDebutSelection		= QTime::currentTime();
  positionDebutSelection	= pos;
  positionFinSelection		= pos;
  _selectionEnCours		= true;
}

void Image::arreterSelection()
{
  if (!selectionEnCours())
    return;

  if (cadreSelectionAffiche())
    effacerCadreSelection();

  _selectionEnCours	= false;
}
/////////////////////////////EVENEMENTS/////////////////////////////////////////
void Image::paintEvent(QPaintEvent *e)
{
  if (!valide())
    {
      erase();
      return;
    }

  calculerImageCourante();

  refresh();
}

void Image::resizeEvent(QResizeEvent *e)
{
  if (!valide())
    return;

  QRect	cadre;
  QSize	s(size());
  float	sx,
    	sy;
  int	largeur,
    	hauteur;

  cadreImageCourante(cadre);
  calculerCoefZoom(sx, sy);

  largeur	= int(s.width() / sx);
  hauteur	= int(s.height() / sy);

  cadre.setWidth(largeur);
  cadre.setHeight(hauteur);

  ajusterImage(cadre, sx, sy, s);
  definirImageCourante(cadre, sx, sy);

  emit imageResized();
}

void Image::mouseDoubleClickEvent(QMouseEvent *e)
{
  switch(outils)
    {
    case 0:
      if (e->button()==LeftButton)
	zoomBy(e->pos(), 2);
      else if (e->button()==RightButton)
	zoomBy(e->pos(), 0.5);
      break;

    case 1:
      break;


    case 2:
      break;

    case 3:
      mouseDoubleClickSelection(e);
      break;

    case 4:
      break;
    
    case 5:
      break;
    }
}

void Image::mousePressEvent(QMouseEvent *e)
{
  selectionInfo.vider();
  switch(outils)
    {case 0:
      /* dans le cas d'une selection	*/
      if (valide())
	{
	  switch(e->button())
	    {
	    case LeftButton:
	      commencerSelection(e->pos());
	      emit coefZoomChange(coefZoom());
	      break;
	      
	    case MidButton:
	      emit coefZoomChange(coefZoom());
	      QPopupMenu *imagemenu=new QPopupMenu;
	      imagemenu->insertItem("Fermer",this,SLOT(fermer()),CTRL+Key_F);
	      imagemenu->insertItem("Charger",this,SLOT(recharger()),CTRL+Key_C);
	      imagemenu->insertSeparator();
	      imagemenu->insertItem("1:4",this,SLOT(zoom1_4()));
	      imagemenu->insertItem("1:3",this,SLOT(zoom1_3()));
	      imagemenu->insertItem("1:2",this,SLOT(zoom1_2()));
	      imagemenu->insertItem("1:1",this,SLOT(zoom1_1()));
	      imagemenu->insertItem("2:1",this,SLOT(zoom2_1()));
	      imagemenu->insertItem("3:1",this,SLOT(zoom3_1()));
	      imagemenu->insertItem("4:1",this,SLOT(zoom4_1()));
	      imagemenu->exec(QCursor::pos());
	      delete imagemenu;
	      break;
	    }
	  break;
	}
	
    case 1:
      {
        selection tmp ;
	selectionPoint(tmp,e->pos(),base);
	if(tmp.premiereVille()!=NULL)
	  {
	    QList<Ville> toselect;
	    if(e->button() == LeftButton)
	      toselect = base.VillesVoisines(tmp.premiereVille());
	    else
	      toselect = base.composante(tmp.premiereVille());
	    selectionCourante.vider();
	    for(int i=0; i<toselect.count(); i++)
	      {
		selectionCourante.inserer(toselect.at(i));
	      }
	    refresh();
	  }
      break;
      }
    case 2:
      _selectionEnCours = TRUE;
      positionDebutSelection=e->pos();
      break;

    case 3:
      if(e->button() == LeftButton)
	mousePressSelection(e);
      else
	mousePressInfo(e);
     
      break;
    
    case 4:
      mousePressCreation(e);
      break;

    case 5:
      mousePressInfo(e);
      break;

    case 6:
      belette_click(e);
      break;
    }
}

void Image::mouseReleaseEvent(QMouseEvent *e)
{
  switch(outils)
    {
    case 0:
      if (selectionEnCours())
	{
	  
	  QRect	cadre;
	  QTime	tempsFinSelection	= QTime::currentTime();
	  
	  cadreSelection(cadre);
	  arreterSelection();
	  
	  if (tempsDebutSelection.msecsTo(tempsFinSelection) < kDelaiMinSelection)
	    return;
	  
	  int	gauche,
	    	droite,
	    	haut,
		bas;

	  zoom(cadre);
	}
	break;

    case 1:
      break;

    case 2:
      {
	int	dx = positionDebutSelection.x() - e->x(),
	  	dy = positionDebutSelection.y() - e->y();
	scroll(dx, dy);
	positionDebutSelection = e->pos();
	_selectionEnCours = FALSE;
      }
      break;

    case 3:
      if(e->button()!=RightButton)
	mouseReleaseSelection(e);
      break;

    case 4:
      mouseReleaseCreation(e);
      break;

    case 5:
      break;
    case 6:
      belette_rilize(e);
      break;
    }
}

void Image::mouseMoveEvent(QMouseEvent *e)
{
  QPoint pepe=e->pos();
  enGlobal(pepe);
  emit positionSourisChangee(pepe.x(), pepe.y());

  switch(outils)
    {
    case 0:
      if (selectionEnCours())
	placerCadreSelection(e->pos());

    case 1:
      break;

    case 2:
      if(_selectionEnCours)
	{
	  int	dx = positionDebutSelection.x() - e->x(),
	  	dy = positionDebutSelection.y() - e->y();
	  scroll(dx,dy);
	  positionDebutSelection = e->pos();
	}
	  break;

    case 3:
      mouseMoveSelection(e);
      break;
    
    case 4:
    mouseMoveCreation(e);
    break;

    case 5:
      break;
  
    }
}

///////////////////////////////////////INITIALISATION DES PARAMETRES/////////////////////
void Image::init(const QPixmap &pmap)
{
  initParametresSelection();
  initParametresZoom(pmap);
}

void Image::initParametresZoom(const QPixmap &pmap)
{
  setMouseTracking(TRUE);
  bool	estErreur	= pmap.isNull() == false,
    	ok		= true;

  etatOk	= false;

  imageSource	= new QPixmap;
  if (!imageSource)
    return;

  imageCourante	= new QPixmap;
  if (!imageCourante)
    {
      delete imageSource;
      return;
    }

  *imageSource	= pmap;
  imageSource->detach();
  if (imageSource->isNull() && estErreur)
    {
      ok	= false;
      warning("Image::initZoomParameters(const QPixmap &): Can't intitialize field 'theImage'");
    }

  if (ok)
    {
      *imageCourante	= pmap;
      imageCourante->detach();
    }
  if (imageCourante->isNull() && estErreur)
    {
      ok	= false;
      imageCourante->resize(0, 0);
      warning("Image::initZoomParameters(const QPixmap &): Can't initialize field 'workingImage'");
    }

  if (ok)
      etatOk		= !pmap.isNull();

  if (etatOk)
    {
      QRect	cadre(0, 0, imageSource->width(), imageSource->height());
      x0		= 0;
      y0		= 0;
      largeurVirtuelle	= cadre.width();
      largeurVirtuelle	= cadre.height();

      largeurReelle	= largeurVirtuelle;
      hauteurReelle	= hauteurVirtuelle;

      definirImageCourante(cadre, 1, 1);
      ajusterWidget();
      resize(imageSource->size());
    }
}

void Image::initParametresSelection()
{
  _selectionEnCours	= false;
  cadreAffiche		= false;
}

//Image2.cpp

#include "Image.h"
#include <stdio.h>
#include "math.h"
#include <qpen.h>


void Image::Initialisation()
{villeOrigine=NULL;
paint=new QPainter; 
paint->begin(this);
selectionOrigine=QPoint(-1,-1);
imcopy=new QPixmap;
imcopy->resize(size());
ppix=new QPainter;
ppix->begin(imcopy);
modDimensions(100,imageSource->height()*100/imageSource->width());
}


//methodes de la selection
void Image::mousePressSelection(QMouseEvent*e)
{ 
 if (LeftButton & e->button())
   {selectionOrigine= e->pos();
   selectionArrivee= e->pos(); 
   paint->setRasterOp(NotROP); 
   paint->drawRect(QRect(selectionOrigine,selectionArrivee));
   }
 }

void Image::mouseMoveSelection(QMouseEvent*e)
{ 
  if ((LeftButton & e->state()) && selectionOrigine!=QPoint(-1,-1))
  { paint->drawRect(QRect(selectionOrigine,selectionArrivee));
  selectionArrivee= e->pos(); 
  paint->drawRect(QRect(selectionOrigine,selectionArrivee));
  }
}

void Image::mouseDoubleClickSelection(QMouseEvent*e)
{ 
  if (LeftButton & e->button()) {
  selection tmp;
  selectionPoint(tmp,e->pos(),base);
  if (tmp.premiereVille()!=NULL) 
    modifierVille(tmp.premiereVille());
  else if (tmp.premiereRoute()!=NULL)
    modifierRoute(tmp.premiereRoute());
}
}


void Image::mouseReleaseSelection(QMouseEvent*e)
{ 
  if ((LeftButton & e->button()) && selectionOrigine!=QPoint(-1,-1))
  {paint->drawRect(QRect(selectionOrigine,selectionArrivee));
  paint->setRasterOp(CopyROP); 
  selectionArrivee= e->pos(); 
  if (dist(selectionOrigine,selectionArrivee)<DISTMIN)
    selectionPoint(selectionCourante,selectionOrigine,base);
  else 
    selectionZone (selectionCourante,QRect(selectionOrigine,selectionArrivee),false, base);
  refresh();  
  selectionOrigine=QPoint(-1,-1);
  }
}



//methodes de la creation

void Image::mousePressCreation(QMouseEvent*e)
{
 
if (LeftButton & e->button())
  {selection tmp ;
    selectionPoint(tmp,e->pos(),base);
  Ville *ob=tmp.premiereVille();
  if(ob==NULL)
    {insererVille(e->pos(),base);
    villeOrigine=NULL;
    refresh();
    }
  else
    {villeOrigine=ob;
    regionVersFenetre(ob->retx(),ob->rety(),selectionOrigine);
    selectionArrivee=e->pos();
    paint->setRasterOp(NotROP);
        QPen pen;
    pen.setWidth(3);
    paint->setPen(pen);
    paint->drawLine(selectionOrigine,selectionArrivee);
    refresh();
    }
  }
}


void Image::mouseMoveCreation(QMouseEvent*e) 
 {if (LeftButton & e->state() & villeOrigine !=NULL)
  { paint->drawLine(selectionOrigine,selectionArrivee);
  selectionArrivee= e->pos();
  selection tmp;
  selectionPoint(tmp,e->pos(),base);
  QPen pen;
  pen.setWidth(3);
  if (tmp.premiereVille()!=NULL) 
    pen.setStyle(SolidLine);
  else
    pen.setStyle(DashDotDotLine);
  paint->setPen(pen);
  paint->drawLine(selectionOrigine,selectionArrivee);
  }
 }


void Image::mouseReleaseCreation(QMouseEvent*e) 
{if (LeftButton & e->button() & villeOrigine !=NULL)
  {selection tmp ;
    paint->drawLine(selectionOrigine,selectionArrivee);
  paint->setRasterOp(CopyROP); 
  selectionPoint(tmp,e->pos(),base);
  Ville* obj=tmp.premiereVille();
  if (obj!=NULL && villeOrigine !=obj)
    {insererRoute(villeOrigine ,obj,base);}
  }
  villeOrigine=NULL; 
  paint->setPen(SolidLine); 
  QPen pen;
  pen.setWidth(1);
  paint->setPen(pen);
}


//methodes des information
void Image::mousePressInfo(QMouseEvent* e)
{if (RightButton & e->button())
  {selection tmp ;
    selectionPoint(tmp,e->pos(),base);
    selectionInfo.vider();
    if(tmp.premiereVille()!=NULL) { 
      //afficherInfo(tmp.premiereVille(),paint);
      selectionInfo.inserer(tmp.premiereVille());
    }
    else if (tmp.premiereRoute()!=NULL) {
      fenetreVersRegion(e->pos(),infoX,infoY);
      //afficherInfo(tmp.premiereRoute(),e->pos(),paint);
      selectionInfo.inserer(tmp.premiereRoute());
    }
    refresh();
  }
}


// insertion/supression/modification des villes/routes
void Image::insererVille(QPoint P, Graphe& b)
{
 uniteCoor x,y;
 fenetreVersRegion(P,x,y);
 Ville* V=new Ville(x,y) ;
 V->modNom("");
 V->modPop(0);
 V->modAlt(0);
 V->modReg("");
 V->modTyp(" ");
 if (DEditVille::edition(V))
   b.inserer(V);}

void Image::insererRoute(Ville* A, Ville* B, Graphe& b)
{Route* R=new Route(A,B);
 R->modNom("");
 R->modLongueur(0);
 R->modType(' ');
 if (DEditRoute::edition(R))
   b.inserer(R);
}

void Image::modifierVille(Ville* V)
{DEditVille::edition(V);}

void Image::modifierRoute(Route*R)
{DEditRoute::edition(R);}

void Image::supprimerVille(Ville* V, Graphe& b)
{b.supprimer(V);}


void Image::supprimerRoute(Route* R,Graphe& b)
{b.supprimer(R);}

void Image::supprimerSelection(selection& s, Graphe& b)
{
  s.premiereVille();
  Ville *v1; Ville *v2;
  while (s.villeCourante()!=NULL) 
    {
      for (v1 = b.premVille() ; v1 != NULL ; v2 = v1 , v1 = b.villeSuiv(v2))
	{
	  if (v1 == s.villeCourante())
	    {
	      b.supprimer(s.villeCourante());
	    }
	}
      s.villeSuivante();
    }

  s.premiereRoute();
  Route *r1; Route *r2;  
  while (s.routeCourante()!=NULL) 
    {
      for (r1 = b.premRoute() ; r1 != NULL ; r2 = r1 , r1 = b.routeSuiv(r2))
	{
	  if (r1 == s.routeCourante())
	    {
	      b.supprimer(s.routeCourante());
	    }
	}
      s.routeSuivante();
    }
}

void Image::supprimerSelectionCourante(){
supprimerSelection(selectionCourante, base);
selectionCourante.vider();
refresh();
}



//affichage de l'etiquette info route/ville

 

void Image::afficherInfo(Ville* V, QPainter* p)
{int mx= width()/2;
int my= height()/2;
QPoint q;
regionVersFenetre(V->retx(),V->rety(),q);
int vx= q.x();
int vy= q.y();
int cx, cy; //coord sup du cadre
int px, py;  //coord point d'attache
if ((vx<mx)&&(vy<my))
  {cx= vx+DECALCADRE;
  cy= vy+DECALCADRE;
  px=cx;
  py=cy;}

else if ((vx>=mx)&&(vy<my))
  {cx=vx -DECALCADRE -LONGCADRE;
  cy=vy+DECALCADRE;
  px=vx -DECALCADRE;
  py=vy +DECALCADRE;}
 
else if ((vx<mx)&&(vy>=my))
  {cx=vx +DECALCADRE;
  cy=vy -DECALCADRE -HTRCADREVILLE;
  px=vx +DECALCADRE;
  py=vy -DECALCADRE;}

 else {cx=vx -DECALCADRE -LONGCADRE;
 cy=vy -DECALCADRE -HTRCADREVILLE;
 px=vx -DECALCADRE;
 py=vy -DECALCADRE;}

p->setPen(QPen(QColor (CADRECOLOR)));
p->drawLine(vx, vy, px, py);
p->fillRect(cx, cy, LONGCADRE, HTRCADREVILLE,QBrush(QColor (CADRECOLOR)));

cy+=5;

p->setPen(QPen(QColor(DONNEECOLOR)));
char tmp[200];
p->drawText(cx+10, cy+10,V->retNom());

sprintf(tmp, "%li",V->retPop());
p->drawText(cx+10, cy+25, QString(tmp)+QString("  hab"));

sprintf(tmp, "%i",V->retAlt());
p->drawText(cx+10, cy+40,QString(tmp)+QString("  m"));

p->drawText(cx+10, cy+55, V->retReg());

switch (V->retTyp()[0])
{case 'c':
  p->drawText(cx+10, cy+70,"capitale");
  break;
case 'r':
  p->drawText(cx+10, cy+70,"capitale regionnale");
  break;
case 'p':
  p->drawText(cx+10, cy+70,"prefecture");
  break;
case 's':
  p->drawText(cx+10, cy+70,"sous prefecture");
  break;
case 'b':
  p->drawText(cx+10, cy+70,"bled paume");
  break;}
}


void Image::afficherInfo(Route* R, uniteCoor x,uniteCoor y, QPainter* p)
{
QPoint P;
regionVersFenetre(x,y,P);
int mx= width()/2;
int my= height()/2;

int cx, cy; //coord sup du cadre
int px, py;  //coord point d'attache
if ((P.x()<mx)&&(P.y()<my))
  {cx= P.x()+DECALCADRE;
  cy= P.y()+DECALCADRE;
  px=cx;
  py=cy;}

else if ((P.x()>=mx)&&(P.y()<my))
  {cx=P.x() -DECALCADRE -LONGCADRE;
  cy=P.y()+DECALCADRE;
  px=P.x() -DECALCADRE;
  py=P.y() +DECALCADRE;}
 
else if ((P.x()<mx)&&(P.y()>=my))
  {cx=P.x() +DECALCADRE;
  cy=P.y() -DECALCADRE -HTRCADREROUTE;
  px=P.x() +DECALCADRE;
  py=P.y() -DECALCADRE;}

 else {cx=P.x() -DECALCADRE -LONGCADRE;
 cy=P.y() -DECALCADRE -HTRCADREROUTE;
 px=P.x() -DECALCADRE;
 py=P.y() -DECALCADRE;}

p->setPen(QPen(QColor (CADRECOLOR)));
p->drawLine(P.x(), P.y(), px, py);
p->fillRect(cx, cy, LONGCADRE, HTRCADREROUTE,QBrush(QColor (CADRECOLOR)));
cy+=5;
p->setPen(QPen(QColor(DONNEECOLOR)));
char tmp[200];
p->drawText(cx+10, cy+10,R->retNom());

sprintf(tmp, "%.3f",R->retLongueur());
p->drawText(cx+10, cy+25,QString(tmp)+QString("  km"));

switch (R->retType())
{case 'a':
  p->drawText(cx+10, cy+40,"autoroute");
  break;
case 'n':
  p->drawText(cx+10, cy+40,"nationale");
  break;
case 'd':
  p->drawText(cx+10, cy+40,"departementale");
  break;
case 'c':
  p->drawText(cx+10, cy+40,"communale");
  break;
case 'h':
  p->drawText(cx+10, cy+40,"chemin carrossable");
  break;}

p->drawText(cx+10, cy+55,(R->retextrem1())->retNom()+QString("-")+ 
	    (R->retextrem2())->retNom());
}

      
//methodes de conversion
void Image::regionVersFenetre(uniteCoor x,uniteCoor y,QPoint& p) {
  //p.setX((int) x);
  //p.setY((int) y);
  p.setX((int) (imageSource->width()*x/largeur));
  p.setY((int) (imageSource->height()*y/hauteur));
  enLocal(p);
}

void Image::fenetreVersRegion(QPoint p,uniteCoor& x,uniteCoor& y) {
  //x=(uniteCoor) p.x();
  //y=(uniteCoor) p.y();
  enGlobal(p);
  x=(uniteCoor) (p.x()*largeur/imageSource->width());
  y=(uniteCoor) (p.y()*hauteur/imageSource->height());
}


//methodes de calcul geometrique
int Image::dist(QPoint a,QPoint b) {
  int x=a.x()-b.x();
  int y=a.y()-b.y();
  return (int) sqrt(x*x+y*y);
}

float Image::dist(int i,int j,int k,int l)
{
  return sqrt( (i-k)*(i-k) + (j-l)*(j-l) );
}

bool Image::presVille(QPoint p1,QPoint p2)
{
  return dist(p1.x(),p1.y(),p2.x(),p2.y())<=DISTMIN;
}

bool Image::presRoute(QPoint p1,QPoint p2,QPoint p3)
{
  QEqua_<uniteCoor> EquaDroite,EquaPerp;
  QPoint_<uniteCoor> Inter;
  QPoint_<uniteCoor> P1(p1.x(),p1.y()),P2(p2.x(),p2.y()),P(p3.x(),p3.y());
  EquaDroite.Droite(P1,P2);
  EquaPerp.PerpendiculaireDroite(P1,P2,P);
  Inter.Intersection(EquaDroite,EquaPerp);
  QRect_<uniteCoor> Rect(P1.x(),P1.y(),P2.x(),P2.y());
  if (Inter.DansRectangle(Rect) && (dist(Inter.x(),Inter.y(),P.x(),P.y()))<=DISTMIN)
  {
    return true;
  }
  return false;
}

//creation de selection
void Image::selectionPoint(selection& S,QPoint P,Graphe& Sel)
{
  S.vider();
  Ville* ville;
  ville=Sel.premVille();
  if (ville!=NULL)
  {
  QPoint pp;
  regionVersFenetre(ville->retx(),ville->rety(),pp);
  while (ville!=NULL && !presVille( pp, P))
  {
    ville=Sel.villeSuiv(ville);
    if (ville!=NULL) regionVersFenetre(ville->retx(),ville->rety(),pp);
  }
  if (ville!=NULL) S.inserer(ville);
  else
  {
    Route* route;
    route=Sel.premRoute();
    if (route!=NULL)
    {
      Ville* V1;
      Ville* V2;
      V1=route->retextrem1();
      V2=route->retextrem2();

      QPoint pp1,pp2;
      regionVersFenetre(V1->retx(),V1->rety(),pp1);
      regionVersFenetre(V2->retx(),V2->rety(),pp2);

      while (route!=NULL && !presRoute( pp1, pp2, P))
      {
        route=Sel.routeSuiv(route);
        if (route!=NULL) { V1=route->retextrem1(); V2=route->retextrem2(); 
        regionVersFenetre(V1->retx(),V1->rety(),pp1);
        regionVersFenetre(V2->retx(),V2->rety(),pp2);}
      }
      if (route!=NULL) S.inserer(route);
    }
  }
  }
}

void Image::selectionPoint(selection& S,QPoint P, selection& Sel)
{
  S.vider();
  Ville* ville;
  ville=Sel.premiereVille();
  if (ville!=NULL)
  {
  QPoint pp;
  regionVersFenetre(ville->retx(),ville->rety(),pp);
  while (ville!=NULL && !presVille( pp, P))
  {
    ville=Sel.villeSuivante();
    if (ville!=NULL) regionVersFenetre(ville->retx(),ville->rety(),pp);
  }
  if (ville!=NULL) S.inserer(ville);
  else
  {
    Route* route;
    route=Sel.premiereRoute();
    if (route!=NULL)
    {
      Ville* V1;
      Ville* V2;
      V1=route->retextrem1();
      V2=route->retextrem2();

      QPoint pp1,pp2;
      regionVersFenetre(V1->retx(),V1->rety(),pp1);
      regionVersFenetre(V2->retx(),V2->rety(),pp2);

      while (route!=NULL && !presRoute( pp1, pp2, P))
      {
        route=Sel.routeSuivante();
        if (route!=NULL) { V1=route->retextrem1(); V2=route->retextrem2(); 
        regionVersFenetre(V1->retx(),V1->rety(),pp1);
        regionVersFenetre(V2->retx(),V2->rety(),pp2);}
      }
      if (route!=NULL) S.inserer(route);
    }
  }
  }
}

void Image::selectionZone(selection& S,QRect r,bool InclStricte,Graphe& Sel)
{
  uniteCoor x1,y1,x2,y2;
  fenetreVersRegion(QPoint(r.x(),r.y()),x1,y1);
  fenetreVersRegion(QPoint(r.width()+r.x(),r.height()+r.y()),x2,y2);

  QRect_<uniteCoor> R(x1,y1,x2,y2);

  S.vider();
  QPoint_<uniteCoor> RouteExtrem1,RouteExtrem2,Inter,V;
  QEqua_<uniteCoor> EquaDiag1,EquaDiag2,EquaRoute;
  Ville* ville;
  ville=Sel.premVille();
  while (ville!=NULL)
  {
    V.Init(ville->retx(),ville->rety());
    if (V.DansRectangle(R))
    {
      S.inserer(ville);
    }
    ville=Sel.villeSuiv(ville);
  }
  QPoint_<uniteCoor> Diag1Extrem1(R.x1(),R.y1());
  QPoint_<uniteCoor> Diag1Extrem2(R.x2(),R.y2());
  QPoint_<uniteCoor> Diag2Extrem1(R.x1(),R.y2());
  QPoint_<uniteCoor> Diag2Extrem2(R.x2(),R.y1());

  EquaDiag1.Droite(Diag1Extrem1,Diag1Extrem2);
  EquaDiag2.Droite(Diag2Extrem1,Diag2Extrem2);

  Route* route;
  route=Sel.premRoute();
  Ville *V1;
  Ville *V2;
  while (route!=NULL)
  {
    V1=route->retextrem1();
    V2=route->retextrem2();
    RouteExtrem1.Init(V1->retx(),V1->rety());
    RouteExtrem2.Init(V2->retx(),V2->rety());
    if (RouteExtrem1.DansRectangle(R) &&  RouteExtrem2.DansRectangle(R))
    // faut arreter la fumee vous voyez des trucs bizarres
    // partout
    {
       S.inserer(route);
    }
    else if (!InclStricte)
    {
      if (!((RouteExtrem1.x()<R.x1() && RouteExtrem2.x()<R.x1()) || (RouteExtrem1.x()>R.x2() && RouteExtrem2.x()>R.x2())
         || (RouteExtrem1.y()<R.y1() && RouteExtrem2.y()<R.y1()) || (RouteExtrem1.y()>R.y2() && RouteExtrem2.y()>R.y2())))
      {
        EquaRoute.Droite(RouteExtrem1,RouteExtrem2);
        if (Inter.Intersection(EquaDiag1,EquaRoute))
        {
          if (Inter.DansRectangle(R)) S.inserer(route);
          else
          {
            if (Inter.Intersection(EquaDiag2,EquaRoute))
            {
              if (Inter.DansRectangle(R)) S.inserer(route);
            }
	  }
        }
      }
    }
    route=Sel.routeSuiv(route);
  }
}

void Image::selectionZone(selection& S,QRect r,bool InclStricte, selection& Sel)
{
  uniteCoor x1,y1,x2,y2;
  fenetreVersRegion(QPoint(r.x(),r.y()),x1,y1);
  fenetreVersRegion(QPoint(r.width()+r.x(),r.height()+r.y()),x2,y2);

  QRect_<uniteCoor> R(x1,y1,x2,y2);

  S.vider();
  QPoint_<uniteCoor> RouteExtrem1,RouteExtrem2,Inter,V;
  QEqua_<uniteCoor> EquaDiag1,EquaDiag2,EquaRoute;
  Ville* ville;
  ville=Sel.premiereVille();
  while (ville!=NULL)
  {
    V.Init(ville->retx(),ville->rety());
    if (V.DansRectangle(R))
    {
      S.inserer(ville);
    }
    ville=Sel.villeSuivante();
  }
  QPoint_<uniteCoor> Diag1Extrem1(R.x1(),R.y1());
  QPoint_<uniteCoor> Diag1Extrem2(R.x2(),R.y2());
  QPoint_<uniteCoor> Diag2Extrem1(R.x1(),R.y2());
  QPoint_<uniteCoor> Diag2Extrem2(R.x2(),R.y1());

  EquaDiag1.Droite(Diag1Extrem1,Diag1Extrem2);
  EquaDiag2.Droite(Diag2Extrem1,Diag2Extrem2);

  Route* route;
  route=Sel.premiereRoute();
  Ville *V1;
  Ville *V2;
  while (route!=NULL)
  {
    V1=route->retextrem1();
    V2=route->retextrem2();
    RouteExtrem1.Init(V1->retx(),V1->rety());
    RouteExtrem2.Init(V2->retx(),V2->rety());
    if (RouteExtrem1.DansRectangle(R) &&  RouteExtrem2.DansRectangle(R))
    {
       S.inserer(route);
    }
    else if (!InclStricte)
    {
      if (!((RouteExtrem1.x()<R.x1() && RouteExtrem2.x()<R.x1()) || (RouteExtrem1.x()>R.x2() && RouteExtrem2.x()>R.x2())
         || (RouteExtrem1.y()<R.y1() && RouteExtrem2.y()<R.y1()) || (RouteExtrem1.y()>R.y2() && RouteExtrem2.y()>R.y2())))
      {
        EquaRoute.Droite(RouteExtrem1,RouteExtrem2);
        if (Inter.Intersection(EquaDiag1,EquaRoute))
        {
          if (Inter.DansRectangle(R)) S.inserer(route);
          else
          {
            if (Inter.Intersection(EquaDiag2,EquaRoute))
            {
              if (Inter.DansRectangle(R)) S.inserer(route);
            }
	  }
        }
      }
    }
    route=Sel.routeSuivante();
  }
}

//dessin du graphe

void Image::DessineGraphe(selection& NS, const QColor& color)
{
  ppix->setPen(QPen(color,4));
  Ville* V_=NS.premiereVille();
  QPoint pp;
  while (V_!=NULL)
  {
    regionVersFenetre(V_->retx(),V_->rety(),pp);
    ppix->drawEllipse(pp.x()-2,pp.y()-2,5,5);
    V_=NS.villeSuivante();

  }
  ppix->setPen(QPen(color,2));
  Route* R_=NS.premiereRoute();
  Ville* V1;
  Ville* V2;
  QPoint pp1,pp2;
  while (R_!=NULL)
  {
    V1=R_->retextrem1();
    V2=R_->retextrem2();
    regionVersFenetre(V1->retx(),V1->rety(),pp1);
    regionVersFenetre(V2->retx(),V2->rety(),pp2);
    
    ppix->drawLine(pp1,pp2); 
    R_=NS.routeSuivante();
  }
}

//rafraichissement de l'ecran

void Image::refresh() {

  ppix->end();
  imcopy->resize(imageCourante->size());
  ppix->begin(imcopy);

   ppix->drawPixmap(0, 0, *imageCourante);

  selectionZone(selectFenActive,QRect(0,0,width(),height()),false,base);
  DessineGraphe(selectFenActive,red);
  DessineGraphe(selectionCourante,yellow);
  //
  if (outils==6) { DessineGraphe(SelectChem2,blue); 
  DessineGraphe(SelectChemin,green);}
  //
  bitBlt( this, 0, 0, imcopy );

  if (outils==3)
    if (selectionInfo.premiereVille()!=NULL)
      afficherInfo(selectionInfo.premiereVille(),paint);
    else if (selectionInfo.premiereRoute()!=NULL)
      afficherInfo(selectionInfo.premiereRoute(),infoX,infoY,paint);
}

void Image::modDimensions(uniteCoor x,uniteCoor y) {
  largeur=x;
  hauteur=y;
}

void Image::imprimer() const
{ QPrinter prt; 
  if ( QPrintDialog::getPrinterSetup( &prt ) ) 
  {  QPainter p; 
     p.begin( &prt ); 
     p.drawPixmap(1,1,*imageCourante);
     p.end(); 
   }       
}   

void Image::exportBase()
{
  QString fn = QFileDialog::getSaveFileName("Base",0,this,"Save File"); 
  base.Sauver(fn);
}

void Image::importBase()
{
  QString fn = QFileDialog::getOpenFileName(0,0,this);
  if(!fn.isEmpty())
    base.Restituer(fn);
}


Graphe* Image::getGraphe()
{
  return &base;
}

void Image::defaire()
{
  base.defaire();
  refresh();
}

void Image::afficheliste (QPainter* paint, QList<RouteVille> *liste)
{
  int index = liste->count();

  for (int i = 0; i < index; i++)     
	liste->at(i)->affiche (ppix);        
}

void Image::calculVoisins()
{
}

void Image::connexiteGraphe()
{
  QList<QList<Ville> > comp= base.listeComposantes();
  cout << "listeComposantes ok." << endl;
}
