#ifndef __Image_h_
#define __Image_h_

#include <qmainwindow.h>
#include <qscrollview.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qprintdialog.h>
#include <qpixmap.h>
#include <qfiledialog.h>

#include <qstring.h>
#include <qkeycode.h>
#include <qdatetime.h>


#include <iostream.h>
#include <limits.h>

#include "selection.h"
#include "Graphe.h"
#include "route.h"
#include "ville.h"
#include "dialogs.h"
#include "geometrie.h"
#include "geometrie.cpp"


#define	kTailleImageMin		20
#define	kDelaiMinSelection	400	// pour reconnaitre les clics d'un double clic
#define	kTailleParDefaut	200	// taille widget quand pas d'image chargee

#define	kMargeCalculImage	10

#define Zoom 0
#define Select 1
#define Deplace 2

/*   OBSOLETE    */
#define	kFacteurZoomMin		0.25F

#define	kFacteurZoomMax		2
/*****************/

class Image : public QWidget
{
 Q_OBJECT
 public:
  Image(QMainWindow *parent=0, const char *name=0);
  Image(const QPixmap &pmap, QWidget *parent=NULL, const char *name=NULL);
  Image(const QString &fileName, QWidget *parent=NULL, const char *name=NULL);

virtual void afficheliste (QPainter*, QList<RouteVille>*); 
void defaire();
void imprimer() const;
void calculVoisins();
void connexiteGraphe();

virtual ~Image();

// l'espace pour stocker les images existe et les pixmaps correspondantes sont
// non nulles
virtual bool valide() const; 

// chargent une nouvelle image source et mettent a jour l'affichage.
// N.B.: si la selection du cadre du zoom est en cours, celle-ci
// est arretee avant. 
virtual void charger(const QPixmap &pmap);
virtual void charger(const QString &nomFichier);
virtual void charger(const char *nomFichier); 

virtual float coefZoom() const;

virtual QPoint position() const;

virtual QSize maximumSize() const;
 public slots:

// reinitialise les transformations effectuees a partir de l'image source.
// L'image courante est alors identique a l'image source.
// N.B.:  si la selection du cadre du zoom est en cours, celle-ci
// est arretee avant.
virtual void recharger();

// fonction de zoom par coefficient d'agrandissement ou de retrecissement. 
// Si l'image  est valide, le zoom est effectue et le point 'pos'  est
// centre dans la widget parente.
// N.B.: le centrage est effectue par l'envoi d'un signal a la widget parente
// si elle existe, qui doit alors se charger de placer correctement l'image;
// sinon le point est centre sur l'ecran. (pas encore implemente) 
virtual void zoomBy(const QPoint &pos, float coef); 
//virtual void zoomBy(const QPoint &pos, int coef, bool zoomAvant);

// fonctions pour zoomer selon un cadre de selection.
// L'agrandissement ou le retrecissement est calcule selon la taille du
// rectangle de selection et la taille courante de l'image.
// si 'recadrer' est vrai, la nouvelle image sera restreinte au contenu
// du cadre de selection.
// N.B.: elles ne font rien si l'image n'est pas valide.
// a l'avenir, elles devraient creer une nouvelle image contenant le
// l'image transformee et ne pas toucher l'image originale, lorsque
// 'recadrer' est vrai. 
virtual void zoom(int x, int y, int w, int h); 
virtual void zoom(const QRect &zoomBox); 

virtual void zoomAbsolu(float coef);

virtual void zoomAbsolu(int x, int y, int w, int h);
virtual void zoomAbsolu(const QRect &zoomBox);

virtual void zoom1_4();
virtual void zoom1_3();
virtual void zoom1_2();
virtual void zoom1_1();
virtual void zoom2_1();
virtual void zoom3_1();
virtual void zoom4_1();

virtual void changeOutils(int);
virtual void outilsSelection();
virtual void outilsZoom();
virtual void outilsDeplace();
virtual void outilsCreation();
virtual void outilsInfo();
virtual void outilsChemin();

virtual void scroll(int dx, int dy);
virtual void scrollAbsolu(int x, int y);

 signals:
void positionSourisChangee(int, int);

void coefZoomChange(float);

void imageScrollee(int x0, int y0);

void tailleMaxChangee(int w, int h);

void imageResized();

protected:
// annuler zoom precedent
 virtual void annulerZoom();

// construit un cadre de taille 's', centre sur 'pos ', contenu dans l'image
// source, et le renvoie dans 'cadre'.
// N.B.: elle ne fait rien si l'image n'est pas valide. 
virtual void cadreCentre(const QPoint &pos, const QSize &s, QRect &cadre) const;

// calcule l'image courante si necessaire, et la place dans 'imageCourante'.
virtual void calculerImageCourante();

// renvoie les coefficients de zoom si l'image est valide.
// Sinon, 'sx' et 'sy' sont mis a 0.
virtual void calculerCoefZoom(float &sx, float &sy) const;

// definit la zone de l'image source a afficher, ainsi que les coefficients de zoom a lui appliquer.
// N.B.: elle ne recalcule pas l'image courante. Ceci est fait par 'paintEvent' qui appelle 'calculerImageCourante'.
virtual void definirImageCourante(const QRect &cadre, float sx, float sy);

// renvoie le cadre de la portion de l'image source affichee si l'image est
//  valide. 'cadre' est inchange sinon.
virtual void cadreImageCourante(QRect &cadre) const;

// renvoie vrai ssi l'image courante a ete recalculee.
// Ceci est fait par l'appel a 'recalculerImageCourante'. 
virtual bool imageCouranteAJour() const; 

/* fonctions pour gerer le selection */

// debute la selection du cadre de zoom en indiquant le point 'pos' comme point de pivot
// du rectangle de selection (celui qui reste fixe).
// N.B.: 'pos' est en coordonnees locales a la widget. 
 virtual void commencerSelection(const QPoint &pos);

 // indique si la selection du cadre de zoom est en cours,
 // ce qui se fait par 'commencerSelection'.
 virtual bool selectionEnCours() const;

 // indique le coin oppose du point de pivot du rectangle de selection,
 // et met a jour l'affichage.
 // N.B.: 'coinOppose' est en coordonnees locales a la widget.
 virtual void placerCadreSelection(const QPoint &coinOppose);	// definit le cadre de selection et m.a.j. affichage 

 // met fin a la selection, et met a jour l'affichage.
 // Le rectangle de selection final peut alors etre recupere par 'cadreSelection'
 // tant qu'une nouvelle selection n'a pas ete lancee.
 virtual void arreterSelection();

 // place dans 'cadre' le rectangle de selection courant.
 // N.B.: celui-ci peut etre recupere apres la fin de la selection
 // tant qu'aucune autre selection n'a ete lancee.
 virtual bool cadreSelection(QRect &cadre) const;

 // affiche et efface le rectangle de selection courant.
 // si p est nul, un nouveau painter sur la widget est ouvert;
 // sinon, l'operation est effectuee dans p.
 // N.B.: comme il n'est pas possible d'ouvrir plus d'un painter sur une
 // meme widget, p doit alors etre un painter sur cette widget pour pouvoir
 // fonctionner correctement.
 virtual void afficherCadreSelection(QPainter *p=NULL);	// idem
 virtual void effacerCadreSelection(QPainter *p=NULL);	// idem

 // transforme les coordonnees de 'pos', locales a la widget, en coordonnees
 // globales i.e. la position du pixel correspondant dans l'image source.
 virtual bool enGlobal(QPoint &pos) const;

 // transforme les coordonnees globales de 'pos' en coordonnees locales a la
 // widget i.e. donne la position dans la widget du pixel de l'image source
 // situe a la position 'pos'.
 virtual bool enLocal(QPoint &pos) const;

 // si l'image est valide, r est ajuste pour tenir dans l'image courante i.e.
 // r est le rectangle forme par l'intersection de lui-meme avec le
 // rectangle qui definit le contenu de l'image;
 // sinon, r est inchange.
 virtual void ajuster(QRect &r) const;

 // si l'image est valide, p est ajuste pour tenir dans l'image courante i.e.
 // p devient est le point du rectangle qui definit le contenu de l'image,
 // le plus proche.
 // sinon, p est inchange.
 virtual void ajuster(QPoint &p) const;

 // ajuste 'cadre' et 's' pour que le rectangle (0, 0, s.width(), s.height())
 // ne contienne que 'cadre' (pas de blanc).
 virtual bool ajusterImage(QRect &cadre, float sx, float sy, QSize &s) const;

 // ajuste la taille max. de la widget.
 // N.B.: ne fait rien si l'image est invalide.
 virtual void ajusterWidget();

 /* evenements */
 virtual void paintEvent(QPaintEvent *);
 virtual void resizeEvent(QResizeEvent *);
 virtual void mouseDoubleClickEvent(QMouseEvent *);
 virtual void mousePressEvent(QMouseEvent *);
 virtual void mouseReleaseEvent(QMouseEvent *);
 virtual void mouseMoveEvent(QMouseEvent *);


private:
 /* fonctions de gestion de la selection */

 // renvoie ssi le cadre de selection est affiche.
 virtual bool cadreSelectionAffiche() const; 

 // restaure le contenu situe sous le cadre de selection;
 // appelee par 'effacerCadreSelection' pour effacer le cadre
 // N.B.: les villes et les routes doivent etre reaffichees a part.
 // De plus, cette fonction ouvre un QPainter sur la widget (...QPainter p(this);...)
 // si p est nul. Comme il n'est pas possible d'ouvrir plus d'un painter sur une
 // meme widget, p doit alors etre un painter sur cette widget pour pouvoir
 // fonctionner correctement.
 virtual void restaurerSousCadreSelection(QPainter *p=NULL);

 /* fonctions d'initialisation */

 void init(const QPixmap &pmap);
 void initParametresZoom(const QPixmap &pmap);
 void initParametresSelection();


 /* empeche la recopie d'une image */
 Image(const Image &) {}
 Image& operator =(const Image &) {}

 private:
 /* attributs */

 // 'imageSource' est l'image chargee: elle n'est pas modifiee tant qu'on ne
 // charge pas une nouvelle image.
 // 'imageCourante' est l'image visible a l'ecran: c'est elle qui subit les
 // transformations.
QPixmap		*imageSource,
  		*imageCourante;

// 'etatOk' est vrai ssi l'image est utilisable i.e.:
// 	- une image a ete chargee avec succes
//	- ???
 bool		etatOk;

 // le point (x0,y0) represente la position du coin superieur gauche de l'image
 // courante dans l'image d'origine;
 // 'largeurVirtuel' et 'hauteurVirtuel' representent la largeur et la hauteur de la portion de l'image affichee dans l'image originale;
 // 'largeurReelle' et 'hauteurReelle' represente la taille reelle de la portion
 // de l'image source affichee.
 int		x0,
   		y0,
   		largeurVirtuelle,
   		hauteurVirtuelle;

 int		largeurReelle,
   		hauteurReelle;

 int		x0Precedent,
   		y0Precedent,
   		largeurVirtuellePrecedente,
   		hauteurVirtuellePrecedente;

 int		largeurReellePrecedente,
   		hauteurReellePrecedente;

 bool		imageAJour;


 // utilise par 'mouseReleaseEvent' pour determiner si le 'mousePressEvent' prededent
 // etait un double clic ou le debut d'une selection.
 QTime		tempsDebutSelection;

 // 'positionDebutSelection' est le point de depart de la selection i.e. le lieu
 // du clic qui a provoque le debut de la selection.
 // 'positionFinSelection' est le coin oppose du rectangle de selection.
 // Ces points sont utilises par toutes les fonctions qui ont besoin de
 // connaitre le rectangle de selection: 'afficherCadreSelection', 'cadreSelection' ,...
 // N.B.: les coordonnees sont exprimees par rapport au coin superieur gauche
 // de la widget
 QPoint		positionDebutSelection,
      		positionFinSelection;

 // indique si une selection est en cours.
 // Cet attribut est utilise par la plupart des fonctions liees a la selection
 // pour savoir si elles peuvent s'executer.
 bool		_selectionEnCours;

 // indique si le cadre de selection est affiche ou non.
 // Il est utilise et mis a jour par 'afficherCadreSelection' et 'effacerCadreSelection'
 // pour savoir si elles peuvent s'executer.
 bool		cadreAffiche;

 // etat
 int outils;

 //-----------------------------------------------------------------------------------------------
//constantes
/////////////

#define DISTMIN 4         //distance minimale entre l'objet et le curseur

// constantes liees a l'affichage de l'etiquette
#define DECALCADRE 30
#define LONGCADRE 200
#define HTRCADREVILLE 85
#define CADRECOLOR white
#define DONNEECOLOR black
#define HTRCADREROUTE 70


public:
virtual void Initialisation();
// conversion fenetre (pixel) <-> region(km)
virtual void regionVersFenetre(uniteCoor, uniteCoor, QPoint&);
virtual void fenetreVersRegion(QPoint,uniteCoor&,uniteCoor&);

virtual void selectionPoint(selection&, QPoint,Graphe&);
virtual void selectionPoint(selection&,QPoint,selection&);
virtual void selectionZone(selection&,QRect, bool,Graphe&);
virtual void selectionZone(selection&, QRect, bool,selection&);

virtual void insererVille(QPoint, Graphe&);
virtual void insererRoute(Ville*, Ville*, Graphe&);
virtual void modifierVille(Ville*);
virtual void modifierRoute(Route*);
virtual void afficherInfo(Ville*, QPainter*);
virtual void afficherInfo(Route*, uniteCoor,uniteCoor, QPainter*);
virtual void supprimerVille(Ville*, Graphe&);
virtual void supprimerRoute(Route*, Graphe&);
virtual void supprimerSelection(selection&, Graphe&);
virtual void supprimerSelectionCourante();

virtual void exportBase();
virtual void importBase();

virtual void DessineGraphe(selection&,const QColor&);

virtual void refresh();

virtual int dist(QPoint,QPoint); 
virtual float dist(int,int,int,int);
virtual bool presVille(QPoint,QPoint);
virtual bool presRoute(QPoint,QPoint,QPoint);

virtual void  modDimensions(uniteCoor,uniteCoor);

 protected:
 
 // evenements

 virtual void mousePressSelection(QMouseEvent*);
 virtual void mouseMoveSelection(QMouseEvent*);
 virtual void mouseReleaseSelection(QMouseEvent*);
 virtual void mouseDoubleClickSelection(QMouseEvent*e);
 
 virtual void mousePressCreation(QMouseEvent*);
 virtual void mouseMoveCreation(QMouseEvent*);
 virtual void mouseReleaseCreation(QMouseEvent*);
 
 virtual void mousePressInfo(QMouseEvent*);
 
private:
 // ville d'origine lors de la creation d'une route
 Ville* villeOrigine;

 //cadre de selection
 QPoint selectionOrigine;
 QPoint selectionArrivee;

 // objets contenus dans la fenetre active
 selection selectFenActive;

 // Chemin 
 selection SelectChemin,SelectChem2;

 // objets contenus dans la selection courante
 selection selectionCourante;

 // objet etiquet
 selection selectionInfo;
 uniteCoor infoX,infoY;

 QPainter 	*paint,    //  qpainter sur la fenetre
                *ppix;	   //  qpainter sur le pixmap de travail

 // base de travail. plus tard ce sera un graphe
 Graphe base;

 // pixmap de travail
 QPixmap *imcopy;

 // largeur et hauteur en km de la carte

 uniteCoor largeur,hauteur;

 // PARTIE CHEMIN
#include "chemin.h"

 public :

   Graphe* getGraphe();

};
#endif // __Image_h_
