/////////////////////////////////////////////////////////////
//
//  cartographie.cpp
//  Derniere modification : 8 Juin 1999
//
/////////////////////////////////////////////////////////////


#include <stdlib.h>

#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qaccel.h>
#include <qmessagebox.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qimage.h>
#include <qfiledialog.h>
#include <qwhatsthis.h>

#include "pixmaps/bipbip.xpm"

#include "cartographie.h"
#include "carte.h"



//  Implementation du constructeur

Cartographie::Cartographie( QMainWindow* parent=0, const char* name=0 ) : QMainWindow( parent, name )
{
  /////////////////////////////////////////////////////////////////////////////////
  // initialisation des donnees privees
  ////////////////////////////////////////////////////////////////////////////////
  pFenetreActive = NULL;
  pCartoToolBar = new QToolBar( this, "Barre d'outils principale" );
  modeActuel = 0;
	
  newCarteText = "Cliquez sur ce bouton pour ouvrir un fichier image comme point de d'une nouvelle carte.";
  fileOpenText = "Cliquez sur ce bouton pour ouvrir une carte existante.";
  fileSaveText = "Cliquez sur ce bouton pour sauvegarder une carte apres avoir effectu des modifications.";
  filePrintText = "Cliquez sur ce bouton pour imprimer la carte affiche  l'ecran.";
  fileCloseText = "Cliquez sur ce bouton pour fermer une carte.";
  cutText = "Cliquez sur ce bouton pour couper quelque chose.";
  copyText = "Cliquez sur ce bouton pour copier quelque chose.";
  pasteText = "Cliquez sur ce bouton pour coller quelque chose.";
  defaireText = "Cliquez sur ce bouton pour annuler la derniere action.";
  refaireText = "Cliquez sur ce bouton pour retablir une action annule.";
  modeZoomText = "Cliquez sur ce bouton pour basculer en mode zoom.\n";
  modeDeplText = "Cliquez sur ce bouton pour basculer en mode deplacement.\n";
  modeCreationText = "Cliquez sur ce bouton pour basculer en mode creation.\n";
  modeSelectText = "Cliquez sur ce bouton pour basculer en mode Selection.\n";
  modeCheminText = "Cliquez sur ce bouton pour basculer en mode chemin.\n";

  /////////////////////////////////////////////////////////////////////////////////
  // definition de la barre de menu
  //////////////////////////////////////////////////////////////////////////////////

  QPopupMenu * fichier = new QPopupMenu();
  QPopupMenu * edition = new QPopupMenu();
  QPopupMenu * gbd = new QPopupMenu();
  QPopupMenu * graphe = new QPopupMenu();
  modes = new QPopupMenu();
  QPopupMenu * aides = new QPopupMenu();

  menuBar()->insertItem( "&Fichier", fichier );
  menuBar()->insertItem( "&Edition", edition );
  menuBar()->insertItem( "&Modes", modes );
  menuBar()->insertItem( "&Villes et routes", gbd );
  menuBar()->insertItem( "&Graphe", graphe );
  menuBar()->insertSeparator();
  menuBar()->insertItem( "&Aide", aides );
  //menuBar()->setSeparator( QMenuBar::InWindowsStyle );

  // definition du menu fichier

  fichier->insertItem( "&Nouvelle carte", this, SLOT(nouvelleCarte()), CTRL+Key_N );
  fichier->insertItem( "&Ouvrir une carte", this, SLOT(ouvrirCarte()), CTRL+Key_O );
  fichier->insertItem( "&Sauver la carte", this, SLOT(sauverCarte()), CTRL+Key_S );
  fichier->insertItem( "&Imprimer la carte", this, SLOT(imprimerCarte()), CTRL+Key_I );
  fichier->insertItem( "&Fermer la carte", this, SLOT(fermerCarte()), CTRL+Key_W );
  fichier->insertSeparator();
  fichier->insertItem( "&Quitter", this, SLOT(quitter()), CTRL+Key_Q );

  // definition du menu edition

  edition->insertItem( "Couper", this, SLOT(couper()), CTRL+Key_X );
  edition->insertItem( "Copier", this, SLOT(copier()), CTRL+Key_C );
  edition->insertItem( "Coller", this, SLOT(coller()), CTRL+Key_V );
  edition->insertSeparator();
  edition->insertItem( "Defaire", this, SLOT(defaire()), CTRL+Key_Z );
  edition->insertItem( "Refaire", this, SLOT(refaire()), SHIFT+CTRL+Key_Z );


  // definition du menu Base de donnees

  gbd->insertItem( "&Exporter la base", this, SLOT(exportBase()) );
  gbd->insertItem( "&Importer la base", this, SLOT(importBase()) );
  gbd->insertSeparator();
  gbd->insertItem( "&Rechercher une ville...", this, SLOT(appelRecherche()) );
  gbd->insertSeparator();
  gbd->insertItem( "&Supprimer", this, SLOT(suppressionBase()), Key_Delete);
  

  // definition du menu graphe

  graphe->insertItem( "&Voisins", this, SLOT(calculVoisins()) );
  graphe->insertItem( "Conne&xite", this, SLOT(connexiteGraphe()) );

  // definition du menu mode

  modes->setCheckable( TRUE );

  menuZoom = modes->insertItem( "&Zoom", this, SLOT(changeModeToZoom()) );
  menuDeplacement = modes->insertItem( "&Deplacement", this, SLOT(changeModeToDeplacement()) );  
  menuCreation = modes->insertItem( "&Creation", this, SLOT(changeModeToCreation()) );
  menuSelection = modes->insertItem( "&Selection", this, SLOT(changeModeToSelection()) );
  menuChemin = modes->insertItem( "C&hemin", this, SLOT(changeModeToChemin()) );


  modes->setItemChecked( menuZoom, TRUE );
  modes->setItemChecked( menuDeplacement, FALSE );
  modes->setItemChecked( menuCreation, FALSE );
  modes->setItemChecked( menuSelection, FALSE );
  modes->setItemChecked( menuChemin, FALSE );

  menuChecked = menuZoom;

  // definition du menu aide

  aides->insertItem( "Sommaire de l'aide", this, SLOT(sommaireAide()), Key_F1 );
  aides->insertItem( "Index des fonctions", this, SLOT(indexAide()) );
  aides->insertSeparator();
  aides->insertItem( "A propos de Roadrunner 99", this, SLOT(aPropos()) );

  /////////////////////////////////////////////////////////////////////////////////
  // definition de la barre d'outils
  /////////////////////////////////////////////////////////////////////////////////

  // ouverture des Pixmaps
  QPixmap PixmapBNC( "pixmaps/filenew.xpm" );  // Bouton nouvelle carte
  QPixmap PixmapBOC( "pixmaps/fileopen.xpm" );  // Bouton ouvrir carte
  QPixmap PixmapBSC( "pixmaps/filesave.xpm" );  // Bouton sauver carte
  QPixmap PixmapBIC( "pixmaps/fileprint.xpm" );  // Bouton imprimer carte
  QPixmap PixmapBFC( "pixmaps/fileclose.xpm" ); // Bouton fermer carte
  QPixmap PixmapCouper( "pixmaps/couper.xpm" );
  QPixmap PixmapCopier( "pixmaps/copier.xpm" );
  QPixmap PixmapColler( "pixmaps/coller.xpm" );
  QPixmap PixmapDefaire( "pixmaps/defaire.xpm" );
  QPixmap PixmapRefaire( "pixmaps/refaire.xpm" );
  QPixmap PixmapZoom( "pixmaps/modezoom.xpm" );
  QPixmap PixmapDeplacement( "pixmaps/modedeplacement.xpm" );
  QPixmap PixmapCreation( "pixmaps/modecreation.xpm" );
  QPixmap PixmapSelection( "pixmaps/modeselection.xpm" );
  QPixmap PixmapChemin( "pixmaps/modechemin.xpm" );

  // Correspondance avec le menu fichier

  QToolButton * pBoutonNouvelleCarte = new QToolButton( PixmapBNC, "Nouvelle carte",
	                                                       0, this, SLOT(nouvelleCarte()), pCartoToolBar );
  QToolButton * pBoutonOuvrirCarte = new QToolButton( PixmapBOC, "Ouvrir une carte",
	                                                    0, this, SLOT(ouvrirCarte()), pCartoToolBar );
  QToolButton * pBoutonSauverCarte = new QToolButton( PixmapBSC, "Sauver la carte",
	                                                    0, this, SLOT(sauverCarte()), pCartoToolBar );
  QToolButton * pBoutonImprimerCarte = new QToolButton( PixmapBIC, "Imprimer la carte",
	                                                    0, this, SLOT(imprimerCarte()), pCartoToolBar );
  QToolButton * pBoutonFermerCarte = new QToolButton( PixmapBFC, "Fermer la carte",
	                                                    0, this, SLOT(fermerCarte()), pCartoToolBar );
  pCartoToolBar->addSeparator();


  // Correspondance avec le menu edition

  QToolButton * pBoutonCouper = new QToolButton( PixmapCouper, "Couper",
	                                                    0, this, SLOT(couper()), pCartoToolBar );
  QToolButton * pBoutonCopier = new QToolButton( PixmapCopier, "Copier",
	                                                    0, this, SLOT(copier()), pCartoToolBar );
  QToolButton * pBoutonColler = new QToolButton( PixmapColler, "Coller",
	                                                    0, this, SLOT(coller()), pCartoToolBar );
  QToolButton * pBoutonDefaire = new QToolButton( PixmapDefaire, "Defaire",
	                                                    0, this, SLOT(defaire()), pCartoToolBar );
  QToolButton * pBoutonRefaire = new QToolButton( PixmapRefaire, "Refaire",
	                                                    0, this, SLOT(refaire()), pCartoToolBar );
  pCartoToolBar->addSeparator();

  // Correpondance avec le menu modes

  pBoutonModeZoom = new QToolButton( PixmapZoom, "Mode Zoom",
						   0, this, SLOT(changeModeToZoom()), pCartoToolBar );
  pBoutonModeZoom->setToggleButton( TRUE );
  //pBoutonModeZoom->setDown( TRUE );
  pBoutonModeZoom->toggle();
  pBoutonChecked = pBoutonModeZoom;

  pBoutonModeDeplacement = new QToolButton( PixmapDeplacement, "Mode Deplacement",
						   0, this, SLOT(changeModeToDeplacement()), pCartoToolBar );
  pBoutonModeDeplacement->setToggleButton( TRUE );


  pBoutonModeCreation = new QToolButton( PixmapCreation, "Mode Creation",
						   0, this, SLOT(changeModeToCreation()), pCartoToolBar );
  pBoutonModeCreation->setToggleButton( TRUE );


  pBoutonModeSelection = new QToolButton( PixmapSelection, "Mode Selection",
						   0, this, SLOT(changeModeToSelection()), pCartoToolBar );
  pBoutonModeSelection->setToggleButton( TRUE );


  pBoutonModeChemin = new QToolButton( PixmapChemin, "Mode Chemin",
						   0, this, SLOT(changeModeToChemin()), pCartoToolBar );
  pBoutonModeChemin->setToggleButton( TRUE ); 

  //definition du What this

  pCartoToolBar->addSeparator();
  
  QWhatsThis::whatsThisButton( pCartoToolBar );

  QWhatsThis::add( pBoutonNouvelleCarte, newCarteText, FALSE );
 
  QWhatsThis::add( pBoutonNouvelleCarte, fileOpenText, FALSE );
  QWhatsThis::add( pBoutonSauverCarte, fileSaveText, FALSE );
  QWhatsThis::add( pBoutonImprimerCarte, filePrintText, FALSE );
  QWhatsThis::add( pBoutonFermerCarte, fileCloseText, FALSE );
  QWhatsThis::add( pBoutonCouper, cutText, FALSE );
  QWhatsThis::add( pBoutonCopier, copyText, FALSE );
  QWhatsThis::add( pBoutonColler, pasteText, FALSE );
  QWhatsThis::add( pBoutonDefaire, defaireText, FALSE );
  QWhatsThis::add( pBoutonRefaire, refaireText, FALSE );
  QWhatsThis::add( pBoutonModeZoom, modeZoomText, FALSE );
  QWhatsThis::add( pBoutonModeDeplacement, modeDeplText, FALSE );
  QWhatsThis::add( pBoutonModeCreation, modeCreationText, FALSE );
  QWhatsThis::add( pBoutonModeSelection, modeSelectText, FALSE );
  QWhatsThis::add( pBoutonModeChemin, modeCheminText, FALSE );


  this->setFixedSize(450, 1);


}


Cartographie::~Cartographie()
{

}

void Cartographie::quitter()
{
	close();
}

void Cartographie::nouvelleCarte()
{

	QString* pFichier = new QString(QFileDialog::getOpenFileName());
	if ( !( pFichier->isNull() ) )
	     {
	       QPixmap* pPixmap;

	       this->import( pFichier, pPixmap );

	       Carte* pCarte = new Carte( pPixmap );
	       pCarte->setCaption( *pFichier );
	
	       pCarte->show();

	       connect( pCarte, SIGNAL(focusPerdu(Carte *)), this, SLOT(fenetreActive(Carte*)) );
	       connect( this, SIGNAL(modeChanged(int)), pCarte, SLOT(changeOutils(int)) );
	       emit modeChanged( modeActuel );

	     }
}

void Cartographie::ouvrirCarte()
{
	QMessageBox::information( this, "Roadrunner 99",
														"Vous avez clique sur Ouvrir une carte\n"
														"Cette fonction n'est pas encore implementee");
}

void Cartographie::sauverCarte()
{
	QMessageBox::information( this, "Roadrunner 99",
														"Vous avez clique sur Sauver une carte\n"
														"Cette fonction n'est pas encore implementee");
}

void Cartographie::imprimerCarte()
{
  if( pFenetreActive != NULL ) pFenetreActive->imprimer();
}

void Cartographie::fermerCarte()
{
	QMessageBox::information( this, "Roadrunner 99",
														"Vous avez clique sur Fermer la carte\n"
														"Cette fonction n'est pas encore implementee");
}

void Cartographie::couper()
{
	QMessageBox::information( this, "Roadrunner 99",
														"Vous avez clique sur Couper\n"
														"Cette fonction n'est pas encore implementee");
}

void Cartographie::copier()
{
	QMessageBox::information( this, "Roadrunner 99",
														"Vous avez clique sur Copier\n"
														"Cette fonction n'est pas encore implementee");
}

void Cartographie::coller()
{
	QMessageBox::information( this, "Roadrunner 99",
														"Vous avez clique sur Coller\n"
														"Cette fonction n'est pas encore implementee");
}

void Cartographie::defaire()
{
  if( pFenetreActive != NULL ) pFenetreActive->defaire();
}

void Cartographie::refaire()
{
	QMessageBox::information( this, "Roadrunner 99",
														"Vous avez clique sur refaire\n"
														"Cette fonction n'est pas encore implementee");
}

void Cartographie::calculVoisins()
{
  if( pFenetreActive != NULL ) pFenetreActive->calculVoisins();
}

void Cartographie::connexiteGraphe()
{  
  if( pFenetreActive != NULL ) pFenetreActive->connexiteGraphe();     
}

void Cartographie::exportBase()
{
  if(pFenetreActive != NULL) pFenetreActive->exportBase(); 
}

void Cartographie::importBase()
{
  if(pFenetreActive != NULL) pFenetreActive->importBase(); 
}

void Cartographie::proprietes()
{
  if ( pFenetreActive != NULL ) pFenetreActive->proprietes();
}

void Cartographie::suppressionBase()
{
  if( pFenetreActive != NULL ) pFenetreActive->supprimerSelectionCourante();
}

void Cartographie::fenetreActive(Carte * pFA)
{
	pFenetreActive = pFA;
}

void Cartographie::changeModeToZoom()
{
  modes->setItemChecked( menuChecked, FALSE );
  modes->setItemChecked( menuZoom, TRUE );

  pBoutonChecked->setOn( FALSE );
  pBoutonModeZoom->setOn( TRUE );

  menuChecked = menuZoom;
  pBoutonChecked = pBoutonModeZoom;

  emit modeChanged( 0 );
  modeActuel = 0;
}

void Cartographie::changeModeToDeplacement()
{
  modes->setItemChecked( menuChecked, FALSE );
  modes->setItemChecked( menuDeplacement, TRUE );

  pBoutonChecked->setOn( FALSE );
  pBoutonModeDeplacement->setOn( TRUE );

  menuChecked = menuDeplacement;
  pBoutonChecked = pBoutonModeDeplacement;

  emit modeChanged( 2 );
  modeActuel = 2;
}

void Cartographie::changeModeToCreation()
{
  modes->setItemChecked( menuChecked, FALSE );
  modes->setItemChecked( menuCreation, TRUE );

  pBoutonChecked->setOn( FALSE );
  pBoutonModeCreation->setOn( TRUE );

  menuChecked = menuCreation;
  pBoutonChecked = pBoutonModeCreation;

  emit modeChanged( 4 );
  modeActuel = 4;
}

void Cartographie::changeModeToSelection()
{
  modes->setItemChecked( menuChecked, FALSE );
  modes->setItemChecked( menuSelection, TRUE );

  pBoutonChecked->setOn( FALSE );
  pBoutonModeSelection->setOn( TRUE );

  menuChecked = menuSelection;
  pBoutonChecked = pBoutonModeSelection;

  emit modeChanged( 3 );
  modeActuel = 3;
}

void Cartographie::changeModeToChemin()
{
  modes->setItemChecked( menuChecked, FALSE );
  modes->setItemChecked( menuChemin, TRUE );

  pBoutonChecked->setOn( FALSE );
  pBoutonModeChemin->setOn( TRUE );

  menuChecked = menuChemin;
  pBoutonChecked = pBoutonModeChemin;

  emit modeChanged( 1 );
  modeActuel = 1;
}


void Cartographie::sommaireAide()
{
  system( "netscape /PROJETS/carto/aidehtml/sommaire.htm &" );
}

void Cartographie::indexAide()
{
  system( "netscape /PROJETS/carto/aidehtml/index.htm &" );
}

void Cartographie::aPropos()
{
  
  QPixmap iconbip( bipbip );
  QMessageBox * PtrMessBox = new QMessageBox(this);
  PtrMessBox->setCaption("A propos de ROAD-RUNNER 99 ...");
  PtrMessBox->setIconPixmap( iconbip );
  PtrMessBox->setText("Le logiciel de cartographie ROAD-RUNNER\n"
		      "a t entirement conu par \n"
		      "les tudiants de LICENCE INFORMATIQUE 99\n"
  		      "dans le cadre d'un projet de fin d'anne\n"
  		      "sous la direction de Frdric KORICHE.\n");
  PtrMessBox->show();
  
}


QString Cartographie::getExtension()
{
  if(chemin->isNull())
    return(NULL);
  const char *c=(const char *)(*chemin);
  int i= strlen(c);
  int j;
  for (j=i-1;c[j]!='.' && j>=0;j--)
    ;
  QString S;
  for (int k=j;k<i;k++)
    {
      S=S+c[k];
    }
  QString *s1=new QString(S);
 
  return *s1;
}


void Cartographie::import(QString * file, QPixmap* &image)
{
  chemin = file;
  if(chemin->isNull())
    return;
  QPixmap* imgSource = new QPixmap;
  QImage* img = new QImage;

  QFile fichier((const char *)chemin);

  QString ext = getExtension();

  if( ( (ext==".jpg" || ext==".jpeg") || ext==".JPG") || ext==".JPEG" )
    {
      QString a("djpeg -gif ");
      QString b(" > /tmp/temp.gif");
      QString command=a+(*chemin)+b;
      system((const char*)command);
      if (!img->load("/tmp/temp.gif"))
	return;
      system("rm /tmp/temp.gif");
    }
  else
    if (!img->load((const char *)(*chemin)))
      return;

  imgSource->convertFromImage(*img);
  delete(img);
  this->reduce256Gray( imgSource );
  image = imgSource;
}


void Cartographie::export( QPixmap* pImage )
{
  if ( chemin == NULL )
    return;

  QPixmap* imgSource = pImage;

  const QString *ex = new QString(getExtension());
  QString ext = *ex;
  if (ext==".jpeg" || ext==".jpg" || ext==".JPG" || ext==".JPEG")
    {
      QFile f("/tmp/image.bmp");
      f.open( IO_WriteOnly );
      QDataStream d(&f);
      d<<*imgSource;
      f.close();
      QString a("cjpeg -outfile "); 
      QString b(" /tmp/.bmp"); 
      QString command=a+(*chemin)+b;
      system((const char*)command);
      system("rm /tmp/image.bmp");
    }
  else {
    if (ext==".bmp" || ext==".BMP")
      {
	imgSource->save((const char *)chemin,"BMP");
      }
    else{
      if (ext==".xpm"){imgSource->save((const char *)chemin,"XPM");}
      else{
	QFile f("/tmp/image.bmp");
	QFile f2("/tmp/image.jpg");
	f.open( IO_WriteOnly );
	f2.open( IO_WriteOnly );
	QDataStream d(&f);
	d<<*imgSource;
	f.close();
	QString a("cjpeg -outfile ");
	QString b(" /tmp/image.bmp");
	QString c(" /tmp/image.jpg");
	QString command=a+c+b;
	system((const char*)command);
	a="djpeg -gif ";
	command=a+c+" > "+(*chemin);
	system((const char*)command);
	system("rm /tmp/image.bmp");
        system("rm /tmp/image.jpg");
      }
    }
  }
}

void Cartographie::exportTo( QPixmap* pImage, QString* pFile )
{
  QString* fn = pFile;
  if ( !fn->isEmpty() )
     {
       chemin = fn;
       return( export( pImage ) );
     }
}


void Cartographie::reduce256Gray( QPixmap* pImage )
{
  int nimax=256;
  QImage *img = new QImage( pImage->convertToImage() );
  img->setNumColors(nimax);
  for (int i=0;i<nimax;i++)
    {
      QColor c=img->color(i);
      QColor ctemp(qGray(c.rgb()),qGray(c.rgb()),qGray(c.rgb()));
      img->setColor(i,ctemp.rgb());
    }
  
  pImage->convertFromImage( *img );
  delete img;
}


void Cartographie::appelRecherche()
{
  if ( pFenetreActive != NULL )
    {
      Graphe* pBase = pFenetreActive->getGraphe();
      pBase->recherche();
    }
}
