//////////////////////////////////////////////////////////
//
//  cartographie.h
// Derniere modification : 8 Juin 1999
//
//////////////////////////////////////////////////////////

#ifndef cartographie_h
#define cartographie_h

#include <qmainwindow.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>

class Carte;

class Cartographie : public QMainWindow
{
		Q_OBJECT
  private :

    // Pointeur sur la derniere carte qui avait le focus
    Carte * pFenetreActive;

    // La barre d'outils et les 5 boutons de mode
    QToolBar * pCartoToolBar;
    QToolButton * pBoutonModeZoom, * pBoutonModeDeplacement,*  pBoutonModeCreation, * pBoutonModeSelection, * pBoutonModeChemin;

    // le menu fichier et ses 5 composantes
    QPopupMenu* modes;
    int menuZoom, menuDeplacement, menuCreation, menuSelection, menuChemin;

    // descripteurs du mode choisi
    // kel mode est actif dans le menu "modes"
    int menuChecked;
    // kel mode est actif dans la toolbar
    QToolButton* pBoutonChecked;
    // kel mode est actif
    int modeActuel;

    // dernier chemin d'acces d'ouverture/chargement
    QString *chemin;

    // textes des bulles d'aide

    char * newCarteText;
    char * fileOpenText;
    char * fileSaveText;
    char * filePrintText;
    char * fileCloseText;
    char * cutText;
    char * copyText;
    char * pasteText;
    char * defaireText;
    char * refaireText;
    char * modeZoomText;
    char * modeDeplText;
    char * modeCreationText;
    char * modeSelectText;
    char * modeCheminText;


  private :
    //extension de l'image.
    virtual QString getExtension();

    //convertit en 256 niveaux de gris.
    virtual void reduce256Gray( QPixmap* );

    // Importer une image quelconque et la transformer en Pixmap
    virtual void import( QString*, QPixmap* &);

    //sauvegarde l'image sur le disque vers chemin. (sauvegarde rapide)
    virtual void export( QPixmap* );

    //sauvegarde l'image sur le disque vers un nouveau chemin.
    virtual void exportTo( QPixmap*, QString* );


  private slots:
	
	// slots pour le menu fichier
		void nouvelleCarte();
		void ouvrirCarte();
		void sauverCarte();
		void imprimerCarte();
		void fermerCarte();
		void quitter();

	// slots pour le menu edition
		void couper();
		void copier();
		void coller();
		void defaire();
		void refaire();

	// slots pour le menu gbd
		void exportBase();
		void importBase();
		void proprietes();
		void suppressionBase();
		void appelRecherche();

	// slots pour le menu graphe
		void calculVoisins();
		void connexiteGraphe();

	// slots pout le menu modes
		void changeModeToZoom();
		void changeModeToDeplacement();
		void changeModeToCreation();
		void changeModeToSelection();
		void changeModeToChemin();

	// slots pour le menu aide
		void sommaireAide();
		void indexAide();
		void aPropos();

	// slots pour recuperer la fenetre "active"
		void fenetreActive(Carte *);

  signals:
	// Indique un changement de modes aux cartes ouvertes
	void modeChanged( int );


  public :  
    Cartographie( QMainWindow* parent=0, const char* name=0 );
    ~Cartographie();

};

#endif

